pro weightedCentroid, x, y, density, xbar, ybar
COMPILE_OPT IDL2, HIDDEN
  d = density

  mass = total(d)

  if mass eq 0 then begin
    mass = n_elements(d)
    d[*] = 1.0
  endif

  xBar = total(x*d)/mass
  yBar = total(y*d)/mass

end

; cubeBinned IS modified by this routine

function adjustrv, rv, rvmodified, binnum, signal, cubebinned, trailer, rvsmooth, cal_valid=cal_valid, region=reg, nosmooth=nosmooth
COMPILE_OPT IDL2

  if n_elements(nosmooth) eq 0 then nosmooth=0

  c = 299792.458D

  sz = size(cubeBinned)
  zsize = sz[1]
  xsize = sz[2]
  ysize = sz[3]

  channelWidth = trailer.xi1 / zsize

;  binnum = long(binnum)

  x = fltarr(xsize, ysize, /nozero)
  y = fltarr(xsize, ysize, /nozero)

  for i=0, ysize-1 do begin
    x[*, i] = findgen(xsize)
    y[*, i] = i
  endfor

  nbins = max(binnum)

  xnode = fltarr(nbins+1)
  ynode = fltarr(nbins+1)
  raies = fltarr(zsize, nbins+1)

  hist = histogram(binnum, reverse_indices=rev)

;  idx = where(rv ne rvmodified)
;  idx2 = where(rvmodified[idx] le -3.1E+38, complement=idx3)

;  goodbinnum = binnum[idx[idx3]]
;  nullbinnum = binnum[idx[idx2]]

;  hist = histogram(goodbinnum, reverse_indices=goodrev)
  
;  error

  ; Par convention, les bins partent a 0
  for i=0, nbins do begin
    interfringe = 0

;    where(binnum eq i)
;    if idx[0] eq -1 then begin
;      print, 'Warning! Unable to find pixels of bin ' + strtrim(i, 2)
;      continue
;    endif

    if rev[i] eq rev[i+1] then begin
      print, 'Error at interation ' + strtrim(i,2)
    endif    
    idx = rev[rev[i]:rev[i+1]-1] ;where(binnum eq i)

    pix = rv[idx]
    pixmodified = rvmodified[idx]

    diff = where(pix ne pixmodified)
;    if diff[0] eq -1 then diff = 0

    if diff[0] ne -1 then begin
;      print, 'Found modified pixel at (' + strtrim(x[idx[diff[0]]], 2) + ', ' + strtrim(y[idx[diff[0]]], 2) + ')'
      test = where(pixmodified[diff] le -3.1e+38)
      if test[0] eq -1 then begin
        v = mean(pixmodified[diff])
      endif else begin
        v = pixmodified[diff[test[0]]]
      endelse

      oldv = pix[diff[0]] 

      if ( ((v gt (oldv - trailer.xik - 0.5*trailer.xik/float(zsize))) and (v lt (oldv - trailer.xik + 0.5*trailer.xik/float(zsize))))  or ((v gt (oldv + trailer.xik - 0.5*trailer.xik/float(zsize))) and (v lt (oldv + trailer.xik + 0.5*trailer.xik/float(zsize)))) ) then begin
;        print, 'Detected interfringe...'
        interfringe = 1
      endif

      if v le -3.1e+38 then begin
        continum = 0
        raie = fltarr(zsize) 
        rvmodified[idx] = -3.1e+38
      endif else begin
        rvmodified[idx] = v
        v = v - trailer.corrv
        z = v / c   ; On approxime... on ne veut que la position gorssiere du barycentre...
        baryLambda = (z+1) * trailer.xl0
        baryPos = (baryLambda - trailer.xl1) / channelWidth
        baryPos = (baryPos lt 0)?zsize + (baryPos mod zsize):baryPos
        baryPos = float(baryPos mod zsize)

        raie = cubeBinned[*, x[idx[diff[0]]], y[idx[diff[0]]]]
        bary = baryc(raie)

;        print, '  Modified barypos found to be ' + strtrim(barypos,2) + ' at (' + strtrim(x[idx[diff[0]]], 2) + ', ' + strtrim(y[idx[diff[0]]], 2) + '), old was ' + strtrim(bary[0],2)

;        error

        if interfringe eq 0 then begin
;          print, 'Interfringe is 0'
          barys = fltarr(zsize, n_elements(bary))
          for j=0, zsize-1 do begin
            b = (baryPos + j) mod zsize
            bary = baryc(raie, float(b))
            barys[j,*] = bary
          endfor

;          print, 'Done all barys'
;          help, barys
;          help, zsize

          BaryPoss = barys[*,0]
          BaryPoss = [baryPoss, baryPoss-zsize, baryPoss+zsize]
          barys = [barys, barys, barys]
          j = min(abs(baryPoss-baryPos), idx2)

;          print, 'Found min at ' + strtrim(idx2,2)

          bary = reform(barys[idx2,*])
;          help, bary
          
        endif else begin
          bary = baryc(raie)          
        endelse

;        print, '  Real barypos is now ' + strtrim(bary[0], 2)

        left = bary[7]-1
        right = bary[8]+1
        left = (left lt 0)?zsize + (left mod zsize):left
        right = right mod zsize

        continum = bary[3]

        if interfringe eq 0 then begin
          if left lt right then begin
            raie[0:left] = continum
            raie[right:zsize-1] = continum
          endif else begin
            raie[right:left] = continum
          endelse
          raie = raie > continum
        endif
     endelse

     raies[*,i] = raie

    endif else begin
      raies[*,i] = cubeBinned[*, x[idx[0]], y[idx[0]]]
    endelse

    for j=0, n_elements(idx)-1 do cubeBinned[*, x[idx[j]], y[idx[j]]] = raies[*,i]

;    help, idx

    weightedCentroid, x[idx], y[idx], signal[idx], xb, yb    

    xnode[i] = xb
    ynode[i] = yb

  endfor

  print, 'Finished...'
  if nosmooth eq 0 then begin
    print, 'Creating smoothed cube...'

    ; Test: on passe la carte rvmodified comme dernier canal du cube
    ; pour faire un smooth rv...
  ;  cubeBinned = [cubeBinned, reform(rvModified, 1, sz[2], sz[3])]

  ;  cubeSmooth = smoothBinnedCube(cubeBinned, binnum, cal_valid=cal_valid, region=reg, withrv=1)

  ;  rvsmooth = reform(cubeSmooth[sz[1], *, *])
  ;  cubeSmooth = cubeSmooth[0:sz[1]-1, *, *]
  ;
  ;  rvsmooth = smoothBinnedCube(reform(rvModified, 1, sz[2], sz[3]), binnum, cal_valid=cal_valid, region=reg, withrv=1)
  ;  rvsmooth = reform(rvsmooth, sz[2], sz[3])

    xmin = floor(min(xNode))>0
    xmax = ceil(max(xNode))<(xsize-1)
    ymin = floor(min(yNode))>0
    ymax = ceil(max(yNode))<(ysize-1)

    triangulate, xnode, ynode, triangles
 
    cubeSmooth = fltarr(zsize, xsize, ysize)

    for i=0, zsize-1 do begin
      cubeSmooth[i, xmin:xmax, ymin:ymax] = trigrid(xnode, ynode, raies[i,*], triangles, nx=xmax-xmin+1, ny=ymax-ymin+1)
    endfor
  endif else begin
    cubeSmooth = cubebinned
  endelse

  return, cubeSmooth
end
