function getfilenames

  files = findfile('scan*.TXT')
  filesgz = findfile('scan*.TXT.gz')

  if(filesgz[0] ne '') then begin
    for i=0, n_elements(filesgz)-1 do begin
      tmp = strsplit(filesgz[i], '.', /extract)
      filesgz[i] = tmp[0] + '.' + tmp[1]
    endfor
  endif

  if(files[0] eq '') then return, filesgz
  if(filesgz[0] eq '') then return, files

  return,  [files, filesgz] ; ['scan1000a4980pas20.TXT', 'scan5000a8980pas20.TXT', 'scan9000a13000pas20-250ms.TXT', 'scan13000a17000pas20-250ms.TXT', 'scan17000a20980pas20-200ms.TXT', 'scan21000a24980pas20-150ms.TXT', 'scan25000a28980pas20-150ms.TXT', 'scan29000a32980pas20-150ms.TXT', 'scan-5000a-3020pas20.TXT', 'scan-3000a980pas20.TXT', 'scan33000a36980pas20-150ms.TXT', 'scan37000a40980pas20-150ms.TXT', 'scan41000a44980pas20-80ms.TXT', 'scan45000a48980pas20-30ms.TXT', 'scan49000a52980pas20-30ms.TXT', 'scan53000a56980pas20-15ms.TXT', 'scan57000a60980pas20-15ms.TXT', 'scan61000a64980pas20-10ms.TXT', 'scan65000a68980pas20-10ms.TXT', 'scan69000a72980pas20-10ms.TXT', 'scan73000a76980pas20-10ms.TXT', 'scan77000a80980pas20-10ms.TXT', 'scan81000a84980pas20-10ms.TXT', 'scan85000a88980pas20-10ms.TXT', 'scan89000a92980pas20-10ms.TXT', 'scan93000a96980pas20-10ms.TXT', 'scan97000a100980pas20-10ms.TXT'] 

end



function get_angle, lambda, period, m
COMPILE_OPT IDL2,HIDDEN

  return, asin(lambda*m/(2.0*period))*180.0/!dpi

end


function get_lambda, thetai, period, m
COMPILE_OPT IDL2,HIDDEN

  return, 2.0*sin(thetai*!dpi/180.0)*period/m

end


pro make_movie

  dataf = getfilenames()

;  dataf = dataf[0]

  fitf = dataf+'-fit.fits'
  dataff = dataf+'.fits.gz'

  imagedir = '/home/odaigle/temp/film_betof/'

  offset = 43.816452

  window, 0, xsize=768, ysize=512

;  id = mpeg_open([768, 512], filename='betof.mpeg', quality=95, iframe_gap=30)
  image = 0

  loadct, 39

  for i=0, n_elements(dataf)-1 do begin
    d = readfits(dataff[i])
    f = readfits(fitf[i])

    tmp = strsplit(dataf[i], 'a', /extract)
    tmp2 = strsplit(tmp[1], 'n', /extract)
    tmp3 = strsplit(tmp[2], 'p', /extract)
    tmp4 = strsplit(tmp[3], 's', /extract)
    tmp4 = strsplit(tmp4[0], '.', /extract)
    tmp4 = strsplit(tmp4[0], '-', /extract)

    theta = float(tmp2[0])
;    thetamax = float(tmp3[0])
    step = float(tmp4[0])

    for j=0, n_elements(f)-1 do begin
      info = file_info(imagedir + strtrim(f[j],2) + '.jpg')
      if (info.exists eq 1) and (info.read eq 1) then continue

      x = reform(d[0,j,*])
      y = reform(d[1,j,*])

      ; Pour avoir un nombre de donnees paires... sinon gaussift chie...
      x = x[0:n_elements(x) - (n_elements(x) mod 2) - 1]
      y = y[0:n_elements(y) - (n_elements(y) mod 2) - 1]

      oplot, x, y

      ; Le fit est meilleur sur y^2...
      fit = gaussfit(x, y*y, a, nterms=4)

      plot, x, y/max(y), xrange=[f[j]-7, f[j]+7], yrange=[0,1.1], xstyle=1, ystyle=1, xtitle='nm', /nodata
      fit = fit > 0
      oplot, x, sqrt(fit)/max(sqrt(fit)), color=64
      oplot, x, y/max(y)
      oplot, [f[j], f[j]], [0, 1.1], linestyle=1
      fwhm = 2.0*sqrt(alog(2.0))*a[2]
      oplot, [f[j]-fwhm, f[j]+fwhm], [0.5, 0.5], color=64
      oplot, [f[j]-fwhm, f[j]-fwhm+0.25], [0.5, 0.52], color=64
      oplot, [f[j]-fwhm, f[j]-fwhm+0.25], [0.5, 0.48], color=64
      oplot, [f[j]+fwhm, f[j]+fwhm-0.25], [0.5, 0.52], color=64
      oplot, [f[j]+fwhm, f[j]+fwhm-0.25], [0.5, 0.48], color=64

      real_theta = offset - theta / 10000.0
      xyouts, f[j]+2, 1.05, textoidl('\theta = ' + strtrim(real_theta, 2) + '^{\circ}')
      theta += step
      xyouts, f[j]+2, 1.0, textoidl('\lambda = ' + strtrim(f[j],2) + ' nm')
      tlambda = get_lambda(real_theta, 0.52, 1.0)*1000.0
      oplot, [tlambda, tlambda], [0.0, 1.1], color=240, linestyle=1
      xyouts, f[j]+2, 0.95, textoidl('\epsilon = ' + strtrim(f[j] - tlambda,2) + ' nm'), color=240
      xyouts, f[j]+2, 0.9, textoidl('FWHM = ' + strtrim(fwhm*2.0,2) + ' nm'), color=64

;      error

      t = tvrd(true=1)
      write_jpeg, imagedir + strtrim(f[j],2) + '.jpg', t, true=1, quality=85

;      error

;      mpeg_put, id, image=t, frame=image

      image += 1


    endfor
  endfor

;  mpeg_save, id, filename='betof.mpeg'
;  mpeg_close, id

end

pro plot_pos, filenames

  ; Il faut que tous les steps soit de
  ; la meme grandeur, sinon on va avoir des problemes...

  n = n_elements(filenames)

  lambda = 0

  for i=0, n-1 do begin
    info = file_info(filenames[i]+'-fit.fits')
    if (info.exists eq 1) and (info.read eq 1) then begin
      print, 'Reading ' + filenames[i]+'-fit.fits'
      data = readfits(filenames[i]+'-fit.fits', /silent)
      lambda[(thetamin[i]-mintheta)/step[0]:(thetamin[i]-mintheta)/step[0]+n_elements(data)-1] = data
      continue
    endif

    info = file_info(filenames[i]+'.fits.gz')
    if (info.exists eq 1) and (info.read eq 1) then begin
      print, 'Reading ' + filenames[i]+'.fits.gz'
      data = readfits(filenames[i]+'.fits.gz', /silent)
    endif else begin
      print, 'Reading ' + filenames[i]
      data = read_spectro(filenames[i])
      writefits, filenames[i]+'.fits', data
      spawn, 'gzip ' + filenames[i]+'.fits'
;      spawn, 'gzip ' + filenames[i]
    endelse

     n_spectres = (size(data))[2]
     n_spectres = n_spectres < 300

;    plot, [thetamin[i], thetamax[i]], [0,65536], /nodata

    for j=0, n_spectres-1 do begin
      x = reform(data[0,j,*])
      y = reform(data[1,j,*])

      ; Pour avoir un nombre de donnees paires... sinon gaussift chie...
      x = x[0:n_elements(x) - (n_elements(x) mod 2) - 1]
      y = y[0:n_elements(y) - (n_elements(y) mod 2) - 1]

      oplot, x, y

      ; Le fit est meilleur sur y^2...
      fit = gaussfit(x, y*y, a, nterms=4)

;      oplot, x, sqrt(fit), linestyle=1

      lambda = [lambda, a[1]]

    endfor
  endfor

  lambda = lambda[1:n_elements(lambda)-1]

  plot, lambda, /ynozero, psym=1

  error

end


function theta_to_consigne, theta
  ; On s'attend d'avoir theta en degres, par en degres*10000.0

  return, round(theta*15774.0*40.0/360.0)


end

pro calibratebetof
COMPILE_OPT IDL2

  filenames = getfilenames()
  
  n = n_elements(filenames)

  thetamin = fltarr(n)
  thetamax = fltarr(n)
  step = fltarr(n)

  for i=0, n-1 do begin
    tmp = strsplit(filenames[i], 'a', /extract)
    tmp2 = strsplit(tmp[1], 'n', /extract)
    tmp3 = strsplit(tmp[2], 'p', /extract)
    tmp4 = strsplit(tmp[3], 's', /extract)
    tmp4 = strsplit(tmp4[0], '.', /extract)
    tmp4 = strsplit(tmp4[0], '-', /extract)

    thetamin[i] = float(tmp2[0])
    thetamax[i] = float(tmp3[0])
    step[i] = float(tmp4[0])

  endfor

  idx = sort(thetamin)
  thetamin = thetamin[idx]
  thetamax = thetamax[idx]
  step = step[idx]
  filenames = filenames[idx]

  ; Il faut que tous les steps soit de
  ; la meme grandeur, sinon on va avoir des problemes...

  mintheta = min(thetamin)

  theta = findgen((max(thetamax)-min(thetamin))/step[0]+1)*step[0]+min(thetamin)
  lambda = fltarr(n_elements(theta))

;  next_should_be = 0.0

  for i=0, n-1 do begin
    info = file_info(filenames[i]+'-fit.fits')
    if (info.exists eq 1) and (info.read eq 1) then begin
      print, 'Reading ' + filenames[i]+'-fit.fits'
      data = readfits(filenames[i]+'-fit.fits', /silent)

     lambda[(thetamin[i]-mintheta)/step[0]:(thetamin[i]-mintheta)/step[0]+n_elements(data)-1] = data
    endif else begin

      info = file_info(filenames[i]+'.fits.gz')
      if (info.exists eq 1) and (info.read eq 1) then begin
        print, 'Reading ' + filenames[i]+'.fits.gz'
        data = readfits(filenames[i]+'.fits.gz', /silent)
      endif else begin
        print, 'Reading ' + filenames[i]
        data = read_spectro(filenames[i])
        writefits, filenames[i]+'.fits', data
        spawn, 'gzip ' + filenames[i]+'.fits'
;        spawn, 'gzip ' + filenames[i]
      endelse

      n_spectres = (size(data))[2]
      n_spectres = n_spectres < 200 ; Quand on a plus de 200 spectres, c'est qu'il y a eu une erreur. Les derniers spectres ne sont pas bons.

;    plot, [thetamin[i], thetamax[i]], [0,65536], /nodata

      for j=0, n_spectres-1 do begin
        x = reform(data[0,j,*])
        y = reform(data[1,j,*])

        ; Pour avoir un nombre de donnees paires... sinon gaussift chie...
        x = x[0:n_elements(x) - (n_elements(x) mod 2) - 1]
        y = y[0:n_elements(y) - (n_elements(y) mod 2) - 1]

        oplot, x, y

        ; Le fit est meilleur sur y^2...
        fit = gaussfit(x, y*y, a, nterms=4)

;        oplot, x, sqrt(fit), linestyle=1

        lambda[(thetamin[i]-mintheta)/step[0]+j] = a[1]
      endfor
      writefits, filenames[i]+'-fit.fits', lambda[(thetamin[i]-mintheta)/step[0]:(thetamin[i]-mintheta)/step[0]+n_spectres-1]
    endelse

;    reg_lambda = lambda[(thetamin[i]-mintheta)/step[0]:(thetamin[i]-mintheta)/step[0]+n_elements(data)-1]
;    der = deriv(reg_lambda)
;    der = mean(der)

;    next_should_be = reg_lambda[n_elements(reg_lambda)-1] + der

;    error

  endfor

  period = 0.520

  theta /= 10000.0

;  plot, theta, lambda, /ynozero, psym=3, xtitle='Degres', ytitle='nm'

  idx = where(theta eq 0)
  
  offset = get_angle(lambda[idx]/1000.0, period, 1.0)

  print, 'Offset is ' + strtrim(offset,2)

  real_theta = offset[0] - theta

  theoretical_lambda = get_lambda(real_theta, period, 1.0)*1000.0

  error = lambda-theoretical_lambda
  derror = abs(error[0:n_elements(error)-2] - error[1:n_elements(error)-1])
  derror = [0, derror]

  std = stddev(derror)
  me = mean(derror)

  idx = where(derror gt me+0.25*std)
  idx = [0, idx]

  offset = 0

;  error

;  window, 0
;  window, 2

  for i=1, n_elements(idx)-2 do begin 
    b = lambda[idx[i]-10:idx[i]-1]
    e = lambda[idx[i]:idx[i]+9]

    coefb = poly_fit(findgen(10), b, 1, yfit=fitb)
    coefe = poly_fit(findgen(10)+10, e, 1, yfit=fite)

    offset = (fitb[n_elements(fitb)-1]-fite[0])+coefb[1]

;    print, offset

;    tvset, 0
;    plot, [b,e], /ynozero
;    oplot, [fitb, fite], psym=1
;    oplot, [fitb, fite+offset], psym=2

    b = idx[i]
    e = i eq (n_elements(idx)-2)?idx[i+1]-1:n_elements(lambda)-1

    lambda[b:e] += offset

;    tvset, 2
;    plot, lambda[0:idx[i+1]-1], /ynozero
;    oplot, [idx[i], idx[i]], [0,1000]

;    error

;    print, 'toto'

  endfor

  error 

  error = lambda-theoretical_lambda

  plot, theta, error, psym=3, xtitle='Lambda', ytitle='lambda error (nm)'

  error = double(error[2500:9699])
  error -= mean(error)
  theta = double(theta[2500:9699])
  consigne = theta_to_consigne(theta)

  fit = svdfit(theta, error, 11, yfit=yfit, /double)

  er = 0.0
  factor = 0.0

  for i=1.0, n_elements(error)  do begin
    f = float(n_elements(error)) / float(i)
    if f ne floor(f) then continue

    fit = [error[findgen(n_elements(error)/i)*i], error[n_elements(error)-1]]

    fit = interpolate(fit, findgen(n_elements(error))/i)

    plot, theta*10000.0, error-fit

    er = [er, max(abs(error-fit))]
    factor = [factor, f]

    print, n_elements(error)/i, er[n_elements(er)-1], max(abs(error-fit)), stddev(abs(error-fit)), er[n_elements(er)-1] + stddev(abs(error-fit))
  endfor

  er = er[1:n_elements(er)-1]
  factor = factor[1:n_elements(factor)-1]
  idx = where(er lt 0.1)
  m = min(factor[idx], idx2)
  factor = factor[idx[idx2]]

  print, 'Needed interpolation factor is ' + strtrim(factor,2)

  ibegin = theta[0]*10000.0
  istep = round((theta[n_elements(theta)-1]-theta[0])*10000.0/factor)

  print, 'Theta begin: ' + strtrim(ibegin,2)
  print, 'Theta step: ' + strtrim(istep,2)
  print, 'N: ' + strtrim(factor,2)

  for i=0, n-1 do begin


  endfor

  

;  error = rebin(error, i)
;  theta = rebin(theta*10000.0, i)



  error






  print, 'tot'



  error


end

