
function getAdaName, cycle, channel
  txtcycle = strtrim(cycle,2)
  if cycle lt 10 then txtcycle = '0' + txtcycle

  txtchannel = strtrim(channel,2)
  if channel lt 10 then txtchannel = '0' + txtchannel

  return, txtcycle + txtchannel

end

function getNextAda, currentCycle, currentChannel, nChannels, nextCycle, nextChannel

  if currentCycle mod 2 eq 0 then begin
    if currentChannel eq 1 then begin
      nextChannel = 1
      nextCycle = currentCycle + 1
    endif else begin
      nextChannel = currentChannel - 1
      nextCycle = currentCycle
    endelse

  endif else begin
    if currentChannel eq nChannels then begin
      nextChannel = nChannels
      nextCycle = currentCycle + 1
    endif else begin
     nextChannel = currentChannel + 1
     nextCycle = currentCycle
    endelse
  endelse

  return, getAdaName(nextCycle, nextChannel)

end


pro plotEverything, raie
  zsize = n_elements(raie)
  sn = computeSignalNoiseValue(raie, signal, noise, flux, continum, b, start, stop, raiet, dispersion) 
  plot, raie, xtitle='Canaux', ytitle='Photons', yrange=[min(raie), max(raie)+2], xrange=[0, zsize-0.5], psym=10, xstyle=1
  xyouts, 100, 450, 'S/N: ' + strtrim(sn,2), /device
  oplot, findgen(zsize), replicate(continum+dispersion, zsize), linestyle=1
  oplot, findgen(zsize), replicate(continum-dispersion, zsize), linestyle=1
  oplot, findgen(zsize), replicate(continum, zsize)
  oplot, [b[0]], [b[1]+continum], psym=1
  oplot, replicate(start, 2), [0,max(raie)], linestyle=1
  oplot, replicate(stop, 2), [0,max(raie)], linestyle=1
;         fit = gaussfit(findgen(sz[1]*10)/10, congrid(cube[*,x,y], sz[1]*10), a, nterms=4, sigma=sigma)
;         oplot, findgen(sz[1]*10)/10, fit, color=120
;         oplot, [a[1]], [a[3]+a[0]/2], color=120, psym=1
  output = baryc(raie)
  fit = makeGaussian([2*output[1], output[0], output[5]/(2*sqrt(2*alog(2))), output[3]], findgen(zsize*10)/10)
  oplot, findgen(zsize*10)/10, fit
end


pro checkSNWhileIntegrating

;   sz = readAdt(channels, cycles, efficiency, qgval, exposure, swvl, qgcte, identification, interferometer, filter, hdronly=1)
   sz = [40, 512, 512]

   window, 0, xsize=sz[1], ysize=sz[2]
   window, 1, xsize=512, ysize=512
   window, 2, xsize=512, ysize=512
   window, 3, xsize=512, ysize=512
   tvset, 0

   cube = fltarr(sz[0], sz[1], sz[2])
   cubetemp = cube
   cubenociel = cube
   sz = size(cube)

   prb = readad2('cal_prb.ad2')
   validCalib = readfits('cal_valid.fits')

   channel = 0
   cycle = 1

   lastx = -1
   lasty = -1

   loadct, 3

   while 1 do begin
     adaname = ''
     toto = getNextAda(cycle, channel, sz[1], nnextCycle, nnextChannel)
     while adaname eq '' do begin
        adaname = getNextAda(nnextCycle, nnextChannel, sz[1], nnnextCycle, nnnextChannel)
        adaname = findfile('[a-zA-Z][0-9][0-9][0-9]' + adaname + '*.[aA][dD][aA]')
        adaname = adaname[0]
        if adaname eq '' then wait, 2
      endwhile

     adaname = getNextAda(cycle, channel, sz[1], nextCycle, nextChannel)
     adaname = findfile('[a-zA-Z][0-9][0-9][0-9]' + adaname + '*.[aA][dD][aA]')
     adaname = adaname[0]
;     help, adaname
;     if adaname eq '' then begin
       cursor, x, y, /device, /nowait
       if (x ne lastx) and (y ne lasty) and (x ne -1) and (y ne -1) then begin
         lastx = x
         lasty = y

         raie = cubenociel[*,x,y]
         raieciel1 = cube[*,x,y]
         if (total(raie) eq 0) and (median(raie) eq 0) and (mean(raie) eq 0) then continue
         tvset, 1
         plotEverything, raie
         xyouts, 10, 10, '1 pixel cursor', /device

         raie = cubenociel[*,(x-1)>0:(x+1)<(sz[2]-1),(y-1)>0:(y+1)<(sz[3]-1)]
         raieciel = cube[*,(x-1)>0:(x+1)<(sz[2]-1),(y-1)>0:(y+1)<(sz[3]-1)]
         sz2 = size(raie)
         raie = reform(raie, sz2[1], sz2[2]*sz2[3])
         raie = total(raie, 2)
         raieciel = reform(raieciel, sz2[1], sz2[2]*sz2[3])
         raieciel = total(raieciel, 2)
         tvset, 2
         plotEverything, raie
         xyouts, 10, 10, strtrim(sz2[2]*sz2[3],2) + ' pixel cursor', /device

         tvset, 3
         plot, raieciel, psym=10
         oplot, raieciel1, psym=10, linestyle=1
         tvset, 0
       endif ;else begin
;         wait, 0.5
;       endelse
;       continue
;     endif

     if adaname eq '' then begin
       wait, 0.5
       continue
     endif

     print, 'Reading ' + adaname
     lastcycle = cycle
     cycle = nextCycle
     channel = nextChannel

     cubetemp += readadatolambda(adaname, channel, sz[1], prb, 1.0, np, 0, 0, silent=1)
     
     if lastcycle ne cycle then begin
       ; Redraw everything...
       cube += cubetemp
       cubenociel = substractSky(cube, validCalib, spectrum=iciel, factors=[0.7, 0.9, 1.0], reg=region, plot=0)
       tvscl, rescale(total(cubenociel,1), 0.01, 0.99)
       xyouts, 10, 10, strtrim(cycle-1,2) + ' cycles displayed', /device
       cubetemp = fltarr(sz[1], sz[2], sz[3])
     endif

   endwhile

end
