function find_intensity, im
  me = median(im)
  return, total(im-me)

  t = total(im,2)
;  m = max(t, idx)
  

  fit = gaussfit(findgen(n_elements(t)), t, a, nterms=4)

;  b = baryc(t)

  return, total(fit)-a[3]*n_elements(fit) ; On retourne le flux
;  monochromatique
;  return, b[2]

end


pro correct_photometry
  
  files = findfile('O*.fits*')
  cen = readad1('cal_cen.ad1')

  dark = readfits('dark.fits', hdrdark)
  dark = dark[0:1499, 0:1499]
  tintdark = sxpar(hdrdark, 'inttime')

  n = n_elements(files)

  im = readfits(files[0], hdrim)
  tintim = sxpar(hdrim, 'inttime')

  im -= (dark*(tintim/tintdark))
  sz = size(im)

  xstar = 0.0
  ystar = 0.0

  nosearch = 0

  info = file_info('xstar.ad1')
  if (info.exists eq 1) and (info.read eq 1) then begin
    xstar = readad1('xstar.ad1')
    ystar = readad1('ystar.ad1')
    nosearch = 1
  endif


  fwhm = 0.0
  intensity = 0.0
  constant = 0.0

  loadct, 0, /silent
  window, 0, xsize=sz[1]/2, ysize=sz[2]/2
  window, 1, xsize=420, ysize=210
  tvset, 0
  tvscl, rescale(rebin(im, sz[1]/2, sz[2]/2), 0.01, 0.99)

  i = 0

  while 1 do begin
    if nosearch ne 1 then begin
      cursor, x, y, /device, /up
      cntrd, im, x*2, y*2, xc, yc, 10

      if xc eq -1 then begin
        print, 'Unable to find a reference star... Try again...'
        continue
      endif

      if (round(xc) eq round(xstar[0])) and (round(yc) eq round(ystar[0])) then break
    endif else begin
      if i eq n_elements(xstar) then break

      xc = xstar[i]
      yc = ystar[i]
      i += 1
    endelse

    loadct, 39, /silent
    tvcircle, 5, xc/2, yc/2, color=240
    loadct, 0, /silent

    tvset, 1
    tvscl, congrid(im[xc-10:xc+10, yc-10:yc+10], 210, 210)
    tvscl, congrid(gauss2dfit(im[xc-10:xc+10, yc-10:yc+10], a, /tilt), 210, 210), 210, 0
    tvset, 0

    fwhm_ = sqrt(a[2]*a[2]+a[3]*a[3])*2.0*sqrt(2.0*alog(2.0))
    toprocess = (median(im[xc-25:xc+25, yc-25:yc+25],4))[5:45, 5:45]
    intensity_ = find_intensity(toprocess)
;    if fwhm_ gt 10 then begin
;      print, 'Star is distorted. Skipping it'
;      continue
;    endif

    print, 'Star is at ' + strtrim(xc,2) + strtrim(yc,2) + ', xfwhm: ' + strtrim(2.0*sqrt(2.0*alog(2.0))*a[2], 2) + ' yfwhm: ' + strtrim(2.0*sqrt(2.8*alog(2.0))*a[3], 2) + ' intensity: ' + strtrim(intensity_,2) + ' constant: ' + strtrim(a[0],2)

    if nosearch ne 1 then begin
      xstar = [xc, xstar]
      ystar = [yc, ystar]
    endif

    fwhm = [fwhm_, fwhm]
    intensity = [intensity_, intensity]
    constant = [a[0], constant]
  endwhile

  if nosearch ne 1 then begin
    xstar = xstar[0:n_elements(xstar)-2]
    ystar = ystar[0:n_elements(ystar)-2]
  endif

  fwhm = fwhm[0:n_elements(fwhm)-2]
  intensity = intensity[0:n_elements(intensity)-2]
  constant = constant[0:n_elements(constant)-2]

  me = median(fwhm)
  std = meanabsdev(fwhm, /median)
  
  idx = where((fwhm ge (me - 6.0*std)) and (fwhm le (me + 6.0*std)))
  if idx[0] eq -1 then begin
    print, 'Found no stars with goof fwhm'
    return
  endif

  xstar = xstar[idx]
  ystar = ystar[idx]
  fwhm = fwhm[idx]
  intensity = intensity[idx]
  constant = constant[idx]

  toto = writead1('xstar.ad1', xstar)
  toto = writead1('ystar.ad1', ystar)

  window, 2
;  plot, [0,n_elements(files)], minmax(intensity), /ylog, /nodata
;  oplot, replicate(0, n_elements(intensity)), intensity, psym=1
  tvset, 0

;  dist_circle, cir, sz[2], cen[0], cen[1]
;  hist = histogram(cir, binsize=4, reverse_indices=r)
;  max = sqrt(cal_cen[0]^2+cal_cen[1]^2)
;  max = max > sqrt((cal_cen[0]-xsize)^2+cal_cen[1]^2)
;  max = max > sqrt((cal_cen[0]-xsize)^2+(cal_cen[1]-ysize)^2)
;  max = max > sqrt(cal_cen[0]^2+(cal_cen[1]-ysize)^2)
;  max -= (max mod 4)
;  max /= 4

;  min = min(cir)
;  min += (4 - (min mod 4))

  intensities = fltarr(n_elements(files), n_elements(intensity))
  constants = intensities
  fwhms = intensities

  intensities[0,*] = intensity
  constants[0,*] = constant
  fwhms[0,*] = fwhm

  for i=1, n_elements(files)-1 do begin
    im = readfits(files[i], imhdr, /silent)
    tintim = sxpar(hdrim, 'inttime')

    im -= (dark*(tintim/tintdark))
    tvscl, rescale(rebin(im, sz[1]/2, sz[2]/2), 0.01, 0.99)
    
    for j=0, n_elements(intensity)-1 do begin
      cntrd, im, xstar[j], ystar[j], xc, yc, 10

      if xc eq -1 then begin
        print, 'Reference star impossible to find. Using old pos...'
        xc = xstar[j]
        yc = ystar[j]
      endif

      loadct, 39, /silent
      tvbox, 20, xc/2, yc/2, color=240
      loadct, 0, /silent

      tvset, 1
      tvscl, congrid(im[xc-20:xc+20, yc-20:yc+20], 210, 210)
;      tvscl, congrid(gauss2dfit(im[xc-20:xc+20, yc-20:yc+20], a, /tilt), 210, 210), 210, 0
      toprocess = (median(im[xc-25:xc+25, yc-25:yc+25],4))[5:45, 5:45]
      tvscl, congrid(toprocess, 210, 210), 210, 0
      tvset, 0
;      help, toprocess      

      intensities[i,j] = find_intensity(toprocess)
;      constants[i,j] = a[0]
;      fwhms[i,j] = sqrt(a[2]*a[2]+a[3]*a[3])*2.0*sqrt(2.0*alog(2.0))

    endfor

    tvset, 2
    plot, [0, i], [-0.2, 0.2], /nodata, ystyle=1
    for j=0, n_elements(intensity)-1 do begin
      oplot, findgen(i+1), reform(intensities[0:i,j])/median(reform(intensities[0:i,j]))-1.0
    endfor
    
    tvset, 0

    wait, 0.5

  endfor

  if n_elements(intensity) ne 1 then begin
    intensities = total(intensities, 2)
  endif

;  intensities /= intensities

  for i=0, n_elements(files)-1 do begin
    print, 'Processing ' + files[i]
    im = readfits(files[i], hdr)
    tintim = sxpar(hdrim, 'inttime')

    im -= (dark*(tintim/tintdark))
    sxaddpar, hdr, 'photcor', 1.0/intensities[i]
    
    filename = strsplit(files[i], '.', /extract)
    if filename[n_elements(filename)-1] eq 'gz' then begin
      idx = strsplit(files[i], '.')
      filename = string((byte(files[i]))[0:idx[n_elements(idx)-1]-2])
    endif else begin
      filename = files[i]
    endelse

    writefits, 'phot-' + filename, im/intensities[i], hdr
    spawn, 'gzip -f ' + 'phot-' + filename    


  endfor



end
