function region, reg, cen
COMPILE_OPT IDL2, HIDDEN

  r = abs(deriv(reg))
  med = median(r)
  candidates = where(r le 10*med)

  if n_elements(candidates) gt (n_elements(reg)-2) then return, [0, n_elements(reg)-1]

  good = candidates - shift(candidates, 1)
  idx = where(good ne 1)

  if idx[0] ne -1 then begin
    start = candidates[idx]
    start = start < cen
    idx = where(start ne cen)
    if idx[0] ne -1 then begin
      start = max(start[idx])
    endif else begin
      start = 0
    endelse
  endif else begin
    start = 0
  endelse

  good = shift(candidates - shift(candidates, 1), -1)
  good = good[0:n_elements(good)-2]

  idx = where(good ne 1)

  if idx[0] ne -1 then begin
    stop = candidates[where(good ne 1)]
    stop = stop > (cen-1)
    idx = where(stop ne (cen-1))
    if idx[0] ne -1 then begin
      stop = min(stop[idx])
    endif else begin
      stop = n_elements(reg)-1
    endelse
  endif else begin
    stop = n_elements(reg)-1
  endelse

  return, [start, stop]
end

function findCalibrationCenter, bru
COMPILE_OPT IDL2

  sz = size(bru)

  cen=[sz[1]/2,sz[2]/2]

  old_dx = !values.f_infinity
  old_dy = !values.f_infinity
  old_xcen = float(cen[0])
  old_ycen = float(cen[1])
  
  dx=10
  dy=10

  iteration = 0

  while (dx ne 0) and (dy ne 0) do begin

    regionx = region(bru[*, (floor(old_ycen)>0)<(sz[2]-1)], (old_xcen>0)<(sz[1]-1))
    regiony = region(bru[(floor(old_xcen)>0)<(sz[1]-1), *], (old_ycen>0)<(sz[2]-1))

    print, 'Regionx: [' + strtrim(regionx[0], 2) + ', ' + strtrim(regionx[1], 2) + ']'
    print, 'Regiony: [' + strtrim(regiony[0], 2) + ', ' + strtrim(regiony[1], 2) + ']'

    fit = poly_fit(findgen(regionx[1]-regionx[0])+regionx[0], bru[regionx[0]:regionx[1]-1, (floor(old_ycen)>0)<(sz[2]-1)], 2)
    xcen = -0.5*fit[1]/fit[2]

;    plot, findgen(regionx[1]-regionx[0]) + regionx[0], bru[regionx[0]:regionx[1]-1, floor(old_ycen)], psym=1
;    oplot, findgen(regionx[1]-regionx[0]) + regionx[0], poly(findgen(regionx[1]-regionx[0]), fit)

    fit = poly_fit(findgen(regiony[1]-regiony[0])+regiony[0], bru[(floor(old_xcen)>0)<(sz[1]-1), regiony[0]:regiony[1]-1], 2, yfit=yfit)
    ycen = -0.5*fit[1]/fit[2]

    ycen = old_ycen lt 0?old_ycen:ycen

    plot, findgen(regiony[1]-regiony[0])+regiony[0], bru[(floor(old_xcen)>0)<(sz[1]-1), regiony[0]:regiony[1]-1], psym=1
    oplot, findgen(regiony[1]-regiony[0])+regiony[0], yfit, color=240
    
;error

    dx = abs(old_xcen-xcen)
    dy = abs(old_ycen-ycen)

    if (dx gt old_dx+(regionx[1]-regionx[0])) or (dy gt old_dy+(regiony[1]-regiony[0])) then begin
      print, 'Error! Centerfind does not converge with center at (' + strtrim(cen[0],2) + ', ' + strtrim(cen[1],2) +')'
      print, dx, dy, old_dx, old_dy
      print, xcen, ycen, old_xcen, old_ycen
      return, -1
    endif

    old_dx = dx
    old_dy = dy
    old_xcen = xcen
    old_ycen = ycen

    print, iteration, dx, dy, xcen, ycen

    iteration = iteration +1
  endwhile

  print, '    Center found at (' + strtrim(xcen,2) + ', ' + strtrim(ycen,2) +')'

  return, [xcen, ycen]

end
