function readAd3, filename, trailer, xyz=re
COMPILE_OPT IDL2
ON_ERROR, 2

  if n_params() eq 0 then begin
    print, 'Syntax:   readAd3(filename, trailer, reorder=[0/1])'
    print, 'filename - ad3 file to read'
    print, ' trailer - array to put ad3 trailer'
    print, ' xyz     - 0 to return data in standard zxy adhow format'
    print, '           1 to return data in xyz format (usually needs a transpose)'
    print, '           default: 0'
    return, -1
  endif

  if n_elements(re) eq 0 then re = 0

  realfilename = filename

  info = file_info(realfilename)

  testgz = strsplit(realfilename, '.', /extract)
  testgz = testgz[n_elements(testgz)-1]

  if (info.exists eq 0) or (info.read eq 0) or (testgz eq 'gz') then begin
    ; On regarde si plutot le fichier est .ad3.gz...
    if (testgz ne 'gz') then begin
      realfilename = filename + '.gz'
      info = file_info(realfilename)
    endif else begin
      realfilename = filename
    endelse

    if (info.exists eq 0) or (info.read eq 0) then return, -1
    if (!version.os_family ne 'unix') then return, -1

    testgz = 'gz'

    openr, unit, realfilename, /get_lun, /compress, /swap_if_big_endian

    spawn, 'gzip -l ' + realfilename, output
    output = strsplit(output[1], ' ', /extract)
    size = ulong64(output[1])
  endif else begin
    openr, unit, realfilename, /get_lun, /swap_if_big_endian
    size = info.size
  endelse

  data = fltarr((size-256)/4, /nozero)

  readu, unit, data

  trailer = {ad3_trailer}

  readu, unit, trailer

  if (trailer.lx*trailer.ly*trailer.lz gt 250*1024*1024) then begin
    print, 'Error: lx or ly or lz seems to be invalid: (' + strtrim(trailer.lx,2) + ', ' + strtrim(trailer.ly,2) + ', ' + strtrim(trailer.lz,2) + ')'
    print, 'If you want to allow arrays as large as this, modify the code!'
    return, -1
  endif

  if trailer.nbdim eq -3 then begin
    data = reform(data, trailer.lx, trailer.ly, trailer.lz)
  endif else begin
    data = reform(data, trailer.lz, trailer.lx, trailer.ly)
  endelse

  free_lun, unit

  if (re eq 1) and (trailer.nbdim eq 3) then begin
    ; return the data ordered in x, y, z
    return, transpose(data, [1, 2, 0])
  endif

  if (re eq 0) and (trailer.nbdim eq -3) then begin
    ; return data ordered in z, x, y
    return, transpose(data, [2, 0, 1])
  endif

  if testgz eq 'gz' then begin
    trailer.was_compressed = 1
  endif else begin
    trailer.was_compressed = 0
  endelse

  return, data

end
