function remove_zero_space, string
  temp = byte(string)

  while temp[0] eq 32 do temp=temp[1:n_elements(temp)-1]
  while temp[n_elements(temp)-1] eq 32 do temp=temp[0:n_elements(temp)-2]

  if strmatch(string, '*.*') ne 0 then begin
    while temp[0] eq byte('0') do temp=temp[1:n_elements(temp)-1]
    while temp[n_elements(temp)-1] eq byte('0') do temp=temp[0:n_elements(temp)-2]

    while temp[n_elements(temp)-1] eq byte('.') do temp=temp[0:n_elements(temp)-2]
  endif

  return, string(temp)

end

function readAdt, channels, cycles, efficiency, qgval, exposure, swvl, qgcte, identification, interferometer, filter, date, time, hdronly=ho

  if n_elements(ho) eq 0 then ho = 0

  adt = file_search('*.adt')

  if n_elements(adt) eq 1 then begin
    adt = file_search('*.ADT')
  endif

  if !version.os_family eq 'Windows' then begin
    spawn, 'type ' + adt, strings
  endif else begin
    spawn, 'cat ' + adt, strings
  endelse

  temp = strings[2]
  temp = strsplit(temp, string(byte([32,9])), /extract)
  date = temp[0]
  time = temp[1]

  temp = where(strmatch(strings, 'Identification*'))
  identification = remove_zero_space((strsplit(strings[temp[0]], '=', /extract))[1])

  temp = where(strmatch(strings, 'Interferometer*'))
  interferometer = (strsplit(strings[temp[0]], '=', /extract))[1]
  if interferometer eq ' PF OM798    (ET50-826)    MARSEILLE2' then interferometer = long(609)
  if interferometer eq ' PF OM600B   (ET50-954)    MARSEILLE4' then interferometer = long(600)

  temp = where(strmatch(strings, 'Filter*'))
  filter = (strsplit(strings[temp[0]], '=', /extract))[1]

  temp = where(strmatch(strings, 'X and Y dimensions*'))
  x = long((strsplit(strings[temp[0]], ' ', /extract))[5])
  y = long((strsplit(strings[temp[0]], ' ', /extract))[6])

  temp = where(strmatch(strings, 'Number of channels*'))
  z = long((strsplit(strings[temp[0]], ' ', /extract))[4])

  temp = where(strmatch(strings, 'Exposure time per channel*'))
  exposure = long((strsplit(strings[temp[0]], ' ', /extract))[5])

  temp = where(strmatch(strings, 'Scanning wavelength*'))
  swvl = float((strsplit(strings[temp], ' ', /extract))[2])

  temp = where(strmatch(strings, 'Queensgate constant*'))
  qgcte = float((strsplit(strings[temp], ' ', /extract))[2])

  temp = where(strmatch(strings, 'Was acquired at*'))

  if ho eq 1 then goto, fin

  tchannels = lonarr(n_elements(temp))
  tcycles = lonarr(n_elements(temp))
  tefficiency = fltarr(n_elements(temp))
  tqgval = lonarr(n_elements(temp))

  for i=0, n_elements(temp)-1 do begin
    temp2 = strsplit(strings[temp[i]], '=', /extract)
    if i eq 0 then begin
      tchannels[i] = long((strsplit(temp2[1], ' ', /extract))[0])
    endif else begin
      tchannels[i] = long((strsplit(temp2[1], ' ', /extract))[0])
    endelse

    tcycles[i] = long((strsplit(temp2[2], ' ', /extract))[0])
    tqgval[i] = long((strsplit(temp2[3], ' ', /extract))[0])

    temp2 = strsplit(strings[temp[i]+1], '=', /extract)
    if n_elements(temp2) gt 4 then begin
      tefficiency[i] = float((strsplit(temp2[3], ' ', /extract))[0])
    endif else tefficiency[i] = 100

  endfor

  tefficiency = tefficiency / 100.0

  maxch = max(tchannels)
  maxcy = max(tcycles)

  channels = intarr(maxch*maxcy)
  cycles = intarr(maxch*maxcy)
  efficiency = fltarr(maxch*maxcy)
  qgval = intarr(maxch*maxcy)

  ; Boucle pour enlever les evenements multiples (du aux suspends)
  for i=1, n_elements(tchannels)-1 do begin
    if tchannels[i] eq 0 then continue

    offset = (tcycles[i]-1)*maxch + tchannels[i]-1
    channels[offset] = tchannels[i]
    cycles[offset] = tcycles[i]
    efficiency[offset] = tefficiency[i]
    qgval[offset] = tqgval[i]
  endfor

  fin:

  return, [z, x, y]

end
