function remove_zero_space, string
COMPILE_OPT IDL2,HIDDEN

  temp = byte(string)

  while temp[0] eq 32 do temp=temp[1:n_elements(temp)-1]
  while temp[n_elements(temp)-1] eq 32 do temp=temp[0:n_elements(temp)-2]

  if strmatch(string, '*.*') ne 0 then begin
    while temp[0] eq byte('0') do temp=temp[1:n_elements(temp)-1]
    while temp[n_elements(temp)-1] eq byte('0') do temp=temp[0:n_elements(temp)-2]

    while temp[n_elements(temp)-1] eq byte('.') do temp=temp[0:n_elements(temp)-2]
  endif

  if temp[0] eq byte('.') then temp = [byte('0'), temp]

  return, string(temp)

end

pro showSpectrum, lambda, adz, oplot=op, f=filename, out=oo, barypos=ba, nonewplot=nonewplot, noclose=noclose, plotsize=plotsize, print=psn, addgauss=addgauss, x_data=x_data, n_spectrum=n_spectrum, nonorm=nonorm, around_median=around_median, _extra=e
ON_ERROR, 0

  if n_elements(op) eq 0 then op=1
  if n_elements(oo) eq 0 then oo='x'
  if n_elements(psn) eq 0 then psn=0

  psn = long(psn)

  if oo eq 'png' then begin
    alternate='png'
    o='x'
  endif else begin
    alternate=''
    o=oo
  endelse

  if n_elements(filename) eq 0 then filename = 'idl'+alternate

  l = readad3(lambda)
  a = adztoskyzone(readadz(adz))

  if n_elements(n_spectrum) ne 0 then a = a[0:n_spectrum-1]

  loadct, 39, /silent

  max = -!values.f_infinity
  min = !values.f_infinity
  
  for i=0, n_elements(a)-1 do begin
    z = l[*, a[i].x1:a[i].x2-1, a[i].y1:a[i].y2-1]
    photon_mean = mean(z)
    sz = size(z)
    if sz[0] ne 1 then begin
      z = reform(z, sz[1], sz[2]*sz[3])
      z = total(z, 2)/(sz[2]*sz[3])
    endif

;    min = min gt min(z)?min(z):min
    if keyword_set(nonorm) eq 0 then z = z - min(z)
    if keyword_set(around_median) then z -= median(z)

    max = max lt max(z)?max(z):max
    min = min gt min(z)?min(z):min
  endfor

;  print, 'Min, max: ' + strtrim(min,2) + ' ' + strtrim(max,2)
  print, 'Mean photon per pixel is ' + strtrim(photon_mean,2)
  print, 'Min: ' + strtrim(min,2) + ', max: ' + strtrim(max,2)

;  print, max
  color_offset=0
  color = (254-color_offset)/n_elements(a)+color_offset
  first_color = color
  step = color - color_offset

  if n_elements(a) eq 1 then first_color=0

  color = first_color

  if (keyword_set(nonewplot) eq 0) then begin

    if (o eq 'x') or (o eq 'z') then begin
      set_plot, o
    endif
    if o eq 'ps' then begin
      if n_elements(plotsize) ne 2 then plotsize=[16,12]
      set_plot, 'ps'
    
      device, /color, bits=8, /encapsulated, file=filename, preview=1, pre_depth=1, pre_xsize=plotsize[0], pre_ysize=plotsize[1], xsize=plotsize[0], ysize=plotsize[1] ;, set_font='-urw-urw bookman l-medium-r-normal--20-0-0-0-p-0-adobe-standard'
    endif
    if o eq 'png' then begin
      set_plot, 'x'
  ;    device, set_character_size=[8,8]
      window, 0
    endif
  endif

  print, 'When plotting, min is ' + strtrim(min,2)

;  plot, [0], [0], yrange=[0, max], xrange=[0, sz[1]-0.5], font=(o eq 'ps'?0:1), ystyle=1, xstyle=1, color=0, background=255, /nodata, _extra=e
  plot, [0], [0], yrange=[min lt 0.0?min*1.1:min/1.1, max*1.1], xrange=(keyword_set(x_data)?minmax(x_data):[0, sz[1]-0.5]), ystyle=1, xstyle=1, color=0, background=255, /nodata, _extra=e

  if n_elements(a) ne 1 then begin
    rgb = bytarr(256,4)

    print, 'Build new colormap'

    tvlct, rgb, /get
    rgb /= 2
    tvlct, rgb
  endif

  for i=0, n_elements(a)-1 do begin
    z = l[*, a[i].x1:a[i].x2-1, a[i].y1:a[i].y2-1]
    sz = size(z)
    if sz[0] ne 1 then begin
      z = reform(z, sz[1], sz[2]*sz[3])
      z = total(z, 2)/(sz[2]*sz[3])
    endif
   if keyword_set(nonorm) eq 0 then  z = z - min(z)
   if keyword_set(around_median) then z -= median(z)

;    print, color

    oplot, keyword_set(x_data)?x_data:findgen(sz[1]), z, color=color, _extra=e

    if keyword_set(addgauss) then begin
      fitg = gaussfit(findgen(n_elements(z)*10.0)/10.0, congrid(z, n_elements(z)*10.0), aaa, nterms=4)
      output = baryc(z)
      fit = makeGaussian([2*output[1]-(computeSignalNoiseValue(z) lt 5.0?output[4]:0), output[0], output[5]/(2*sqrt(2*alog(2))), output[3]], findgen(n_elements(z)*10)/10.0)
      oplot, findgen(n_elements(z)*10.0)/10.0, fit, color=color, _extra=e
      oplot, findgen(n_elements(z)*10.0)/10.0, fitg, color=color, linestyle=1, _extra=e
    endif

    color = color + step

  endfor

  if keyword_set(ba) then begin
    color = first_color
    for i=0, n_elements(a)-1 do begin
      z = rebin(l[*, a[i].x1:a[i].x2-1, a[i].y1:a[i].y2-1], sz[1], 1, 1)
      sn=computeSignalNoiseValue(z, signal, noise, flux, continum, b, start, stop, raiet, dispersion)
;      print, start, stop
      oplot, [b[0]], [b[1]-min+continum], psym=1, color=color, _extra=e
      oplot, [start eq 0?sz[1]-1:start, start eq 0?sz[1]-1:start], [z[start]-max/8.0-min, z[start]+max/8.0-min], color=color, linestyle=1, _extra=e
      oplot, [stop eq 0?sz[1]-1:stop, stop eq 0?sz[1]-1:stop], [z[stop]-max/8.0-min, z[stop]+max/8.0-min], color=color, linestyle=1, _extra=e
      oplot, [0, sz[1]], [continum-min, continum-min], color=color, _extra=e
      oplot, [0, sz[1]], [continum-min-dispersion, continum-min-dispersion], color=color, linestyle=1, _extra=e
      oplot, [0, sz[1]], [continum-min+dispersion, continum-min+dispersion], color=color, linestyle=1, _extra=e
;      oplot, [start lt stop?start:stop, start lt stop?stop:start], [continum-min, continum-min], color=color, _extra=e
;      print, start, stop
;      print, z[st
;      print, [z[start]-max/8.0, z[start]+max/8.0]
;      print, [z[stop]-max/8.0, z[stop]+max/8.0]
      color = color+step
    endfor
  endif

  if psn ne 0 then begin
    color = first_color
    xpos = float(sz[1])/20
    ypos = max*0.9*1.1
    if o ne 'ps' then begin
      ystep = max*0.025*1.1
    endif else begin
      ystep = max*0.05*1.1
    endelse

    for i=0, n_elements(a)-1 do begin
      sn=computeSignalNoiseValue(rebin(l[*, a[i].x1:a[i].x2-1, a[i].y1:a[i].y2-1], sz[1], 1, 1), signal, noise, flux, continum, b, start, stop, raiet, dispersion, fwhm)
      if (psn and 1) eq 1 then begin
        xyouts, xpos, ypos, 'S/N: ' + remove_zero_space(strtrim((round(sn*100.0))/100.0,2)), color=color, charsize=0.8
        ypos -= ystep
      endif
      if (psn and 2) eq 2 then begin
        xyouts, xpos, ypos, 'Barypos: ' + remove_zero_space(strtrim(round(b[0]*100.0)/100.0,2)), color=color, charsize=0.8
        ypos -= ystep
      endif
      if (psn and 16) eq 16 then begin
        xyouts, xpos, ypos, 'N: ' + remove_zero_space(strtrim(round(flux*100.0)/100.0,2)), color=color, charsize=0.8
        ypos -= ystep
      endif
      if (psn and 4) eq 4 then begin
        xyouts, xpos, ypos, 'Dispersion: ' + remove_zero_space(strtrim(round(dispersion*100.0)/100.0,2)), color=color, charsize=0.8
        ypos -= ystep
      endif
      if (psn and 8) eq 8 then begin
        xyouts, xpos, ypos, 'FWHM: ' + remove_zero_space(strtrim(round(fwhm*100.0)/100.0,2)), color=color, charsize=0.8
        ypos -= ystep
      endif

      ypos -= ystep/4.0
      color = color + step
    endfor
  endif


  print, 'Writing ' + filename

  if alternate eq 'png' then begin
    toto = tvrd(true=1)
;    sz = size(toto)
;    toto = congrid(toto, sz[1], sz[2]*4, sz[3]*4)
;    for i=0, sz[1]-1 do begin
;      toto[i,*,*] = smooth(toto[i,*,*], 2)
;    endfor
;    toto = rebin(toto, 3, sz[2], sz[3])
;    tv, toto, true=1
;    error
    write_png, filename, toto
  endif


  if (o eq 'ps') and (keyword_set(noclose) eq 0) then device, /close

  rmserror, lambda, adz

  print, 'SNR: ' + strtrim(computeSignalNoiseValue(rebin(l[*, a[0].x1:a[0].x2-1, a[0].y1:a[0].y2-1], sz[1], 1, 1), signal, noise, flux, continum, b, start, stop, raiet, dispersion),2)
  print, 'Dispersion: ' + strtrim(dispersion,2)
  print, 'Barycenter: ' + strtrim(b[0],2) + ', ' + strtrim(b[1],2)
  print, 'Flux: ' + strtrim(flux,2)
  print, ''

end
